import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:image_editor_plus/modules/color_filter_generator.dart';

///The [ColorFilterSequence] class to define a Filter which will applied to each color, consists of multiple [SubFilter]s
class ColorFilterSequence {
  String name;
  List<List<double>> subFilters;
  ColorFilterSequence({required this.name}) : subFilters = [];

  Widget apply(Widget child) {
    Widget tree = child;

    for (int i = 0; i < subFilters.length; i++) {
      tree = ColorFiltered(
        colorFilter: ColorFilter.matrix(subFilters[i]),
        child: tree,
      );
    }

    return tree;
  }

  void addSubFilter(List<double> subFilter) {
    subFilters.add(subFilter);
  }

  void addSubFilters(List<List<double>> subFilters) {
    this.subFilters.addAll(subFilters);
  }
}

// NoFilter: No filter
class NoFilter extends ColorFilterSequence {
  NoFilter() : super(name: "No Filter");
}

// Clarendon: adds light to lighter areas and dark to darker areas
class ClarendonFilter extends ColorFilterSequence {
  ClarendonFilter() : super(name: "Clarendon") {
    addSubFilters([
      ColorFilterGenerator.brightness(.1),
      ColorFilterGenerator.contrast(.1),
      ColorFilterGenerator.saturation(.15),
    ]);
  }
}

class AddictiveRedFilter extends ColorFilterSequence {
  AddictiveRedFilter() : super(name: "AddictiveRed") {
    addSubFilters([
      ColorFilterGenerator.addictiveColor(50, 0, 0),
    ]);
  }
}

class AddictiveBlueFilter extends ColorFilterSequence {
  AddictiveBlueFilter() : super(name: "AddictiveBlue") {
    addSubFilters([
      ColorFilterGenerator.addictiveColor(0, 0, 50),
    ]);
  }
}

// Gingham: Vintage-inspired, taking some color out
class GinghamFilter extends ColorFilterSequence {
  GinghamFilter() : super(name: "Gingham") {
    addSubFilters([
      ColorFilterGenerator.sepia(.04),
      ColorFilterGenerator.contrast(-.15),
    ]);
  }
}

// Moon: B/W, increase brightness and decrease contrast
class MoonFilter extends ColorFilterSequence {
  MoonFilter() : super(name: "Moon") {
    addSubFilters([
      ColorFilterGenerator.grayscale(),
      ColorFilterGenerator.contrast(-.04),
      ColorFilterGenerator.brightness(0.1),
    ]);
  }
}

// Lark: Brightens and intensifies colours but not red hues
class LarkFilter extends ColorFilterSequence {
  LarkFilter() : super(name: "Lark") {
    addSubFilters([
      ColorFilterGenerator.brightness(0.08),
      ColorFilterGenerator.grayscale(),
      ColorFilterGenerator.contrast(-.04),
    ]);
  }
}

// Reyes: a vintage filter, gives your photos a “dusty” look
class ReyesFilter extends ColorFilterSequence {
  ReyesFilter() : super(name: "Reyes") {
    addSubFilters([
      ColorFilterGenerator.sepia(0.4),
      ColorFilterGenerator.brightness(0.13),
      ColorFilterGenerator.contrast(-.05),
    ]);
  }
}

// Juno: Brightens colors, and intensifies red and yellow hues
class JunoFilter extends ColorFilterSequence {
  JunoFilter() : super(name: "Juno") {
    addSubFilters([
      ColorFilterGenerator.rgbScale(1.01, 1.04, 1),
      ColorFilterGenerator.saturation(0.3),
    ]);
  }
}

// Slumber: Desaturates the image as well as adds haze for a retro, dreamy look – with an emphasis on blacks and blues
class SlumberFilter extends ColorFilterSequence {
  SlumberFilter() : super(name: "Slumber") {
    addSubFilters([
      ColorFilterGenerator.brightness(.1),
      ColorFilterGenerator.saturation(-0.5),
    ]);
  }
}

// Crema: Adds a creamy look that both warms and cools the image
class CremaFilter extends ColorFilterSequence {
  CremaFilter() : super(name: "Crema") {
    addSubFilters([
      ColorFilterGenerator.rgbScale(1.04, 1, 1.02),
      ColorFilterGenerator.saturation(-0.05),
    ]);
  }
}

// Ludwig: A slight hint of desaturation that also enhances light
class LudwigFilter extends ColorFilterSequence {
  LudwigFilter() : super(name: "Ludwig") {
    addSubFilters([
      ColorFilterGenerator.brightness(.05),
      ColorFilterGenerator.saturation(-0.03),
    ]);
  }
}

// Aden: This filter gives a blue/pink natural look
class AdenFilter extends ColorFilterSequence {
  AdenFilter() : super(name: "Aden") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(228, 130, 225, 0.13),
      ColorFilterGenerator.saturation(-0.2),
    ]);
  }
}

// Perpetua: Adding a pastel look, this filter is ideal for portraits
class PerpetuaFilter extends ColorFilterSequence {
  PerpetuaFilter() : super(name: "Perpetua") {
    addSubFilters([
      ColorFilterGenerator.rgbScale(1.05, 1.1, 1),
    ]);
  }
}

// Amaro: Adds light to an image, with the focus on the centre
class AmaroFilter extends ColorFilterSequence {
  AmaroFilter() : super(name: "Amaro") {
    addSubFilters([
      ColorFilterGenerator.saturation(0.3),
      ColorFilterGenerator.brightness(0.15),
    ]);
  }
}

// Mayfair: Applies a warm pink tone, subtle vignetting to brighten the photograph center and a thin black border
class MayfairFilter extends ColorFilterSequence {
  MayfairFilter() : super(name: "Mayfair") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(230, 115, 108, 0.05),
      ColorFilterGenerator.saturation(0.15),
    ]);
  }
}

// Rise: Adds a "glow" to the image, with softer lighting of the subject
class RiseFilter extends ColorFilterSequence {
  RiseFilter() : super(name: "Rise") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 170, 0, 0.1),
      ColorFilterGenerator.brightness(0.09),
      ColorFilterGenerator.saturation(0.1),
    ]);
  }
}

// Hudson: Creates an "icy" illusion with heightened shadows, cool tint and dodged center
class HudsonFilter extends ColorFilterSequence {
  HudsonFilter() : super(name: "Hudson") {
    addSubFilters([
      ColorFilterGenerator.rgbScale(1, 1, 1.25),
      ColorFilterGenerator.contrast(0.1),
      ColorFilterGenerator.brightness(0.15),
    ]);
  }
}

// Valencia: Fades the image by increasing exposure and warming the colors, to give it an antique feel
class ValenciaFilter extends ColorFilterSequence {
  ValenciaFilter() : super(name: "Valencia") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 225, 80, 0.08),
      ColorFilterGenerator.saturation(0.1),
      ColorFilterGenerator.contrast(0.05),
    ]);
  }
}

// X-Pro II: Increases color vibrance with a golden tint, high contrast and slight vignette added to the edges
class XProIIFilter extends ColorFilterSequence {
  XProIIFilter() : super(name: "X-Pro II") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 255, 0, 0.07),
      ColorFilterGenerator.saturation(0.2),
      ColorFilterGenerator.contrast(0.15),
    ]);
  }
}

// Sierra: Gives a faded, softer look
class SierraFilter extends ColorFilterSequence {
  SierraFilter() : super(name: "Sierra") {
    addSubFilters([
      ColorFilterGenerator.contrast(-0.15),
      ColorFilterGenerator.saturation(0.1),
    ]);
  }
}

// Willow: A monochromatic filter with subtle purple tones and a translucent white border
class WillowFilter extends ColorFilterSequence {
  WillowFilter() : super(name: "Willow") {
    addSubFilters([
      ColorFilterGenerator.grayscale(),
      ColorFilterGenerator.colorOverlay(100, 28, 210, 0.03),
      ColorFilterGenerator.brightness(0.1),
    ]);
  }
}

// Lo-Fi: Enriches color and adds strong shadows through the use of saturation and "warming" the temperature
class LoFiFilter extends ColorFilterSequence {
  LoFiFilter() : super(name: "Lo-Fi") {
    addSubFilters([
      ColorFilterGenerator.contrast(0.15),
      ColorFilterGenerator.saturation(0.2),
    ]);
  }
}

// Inkwell: Direct shift to black and white
class InkwellFilter extends ColorFilterSequence {
  InkwellFilter() : super(name: "Inkwell") {
    addSubFilters([
      ColorFilterGenerator.grayscale(),
    ]);
  }
}

// Hefe: Hight contrast and saturation, with a similar effect to Lo-Fi but not quite as dramatic
class HefeFilter extends ColorFilterSequence {
  HefeFilter() : super(name: "Hefe") {
    addSubFilters([
      ColorFilterGenerator.contrast(0.1),
      ColorFilterGenerator.saturation(0.15),
    ]);
  }
}

// Nashville: Warms the temperature, lowers contrast and increases exposure to give a light "pink" tint – making it feel "nostalgic"
class NashvilleFilter extends ColorFilterSequence {
  NashvilleFilter() : super(name: "Nashville") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(220, 115, 188, 0.12),
      ColorFilterGenerator.contrast(-0.05),
    ]);
  }
}

// Stinson: washing out the colors ever so slightly
class StinsonFilter extends ColorFilterSequence {
  StinsonFilter() : super(name: "Stinson") {
    addSubFilters([
      ColorFilterGenerator.brightness(0.1),
      ColorFilterGenerator.sepia(0.3),
    ]);
  }
}

// Vesper: adds a yellow tint that
class VesperFilter extends ColorFilterSequence {
  VesperFilter() : super(name: "Vesper") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 225, 0, 0.05),
      ColorFilterGenerator.brightness(0.06),
      ColorFilterGenerator.contrast(0.06),
    ]);
  }
}

// Earlybird: Gives an older look with a sepia tint and warm temperature
class EarlybirdFilter extends ColorFilterSequence {
  EarlybirdFilter() : super(name: "Earlybird") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 165, 40, 0.2),
      ColorFilterGenerator.saturation(0.15),
    ]);
  }
}

// Brannan: Increases contrast and exposure and adds a metallic tint
class BrannanFilter extends ColorFilterSequence {
  BrannanFilter() : super(name: "Brannan") {
    addSubFilters([
      ColorFilterGenerator.contrast(0.2),
      ColorFilterGenerator.colorOverlay(140, 10, 185, 0.1),
    ]);
  }
}

// Sutro: Burns photo edges, increases highlights and shadows dramatically with a focus on purple and brown colors
class SutroFilter extends ColorFilterSequence {
  SutroFilter() : super(name: "Sutro") {
    addSubFilters([
      ColorFilterGenerator.brightness(-0.1),
      ColorFilterGenerator.saturation(-0.1),
    ]);
  }
}

// Toaster: Ages the image by "burning" the centre and adds a dramatic vignette
class ToasterFilter extends ColorFilterSequence {
  ToasterFilter() : super(name: "Toaster") {
    addSubFilters([
      ColorFilterGenerator.sepia(0.1),
      ColorFilterGenerator.colorOverlay(255, 145, 0, 0.2),
    ]);
  }
}

// Walden: Increases exposure and adds a yellow tint
class WaldenFilter extends ColorFilterSequence {
  WaldenFilter() : super(name: "Walden") {
    addSubFilters([
      ColorFilterGenerator.brightness(0.1),
      ColorFilterGenerator.colorOverlay(255, 255, 0, 0.2),
    ]);
  }
}

// 1977: The increased exposure with a red tint gives the photograph a rosy, brighter, faded look.
class F1977Filter extends ColorFilterSequence {
  F1977Filter() : super(name: "1977") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 25, 0, 0.15),
      ColorFilterGenerator.brightness(0.1),
    ]);
  }
}

// Kelvin: Increases saturation and temperature to give it a radiant "glow"
class KelvinFilter extends ColorFilterSequence {
  KelvinFilter() : super(name: "Kelvin") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 140, 0, 0.1),
      ColorFilterGenerator.rgbScale(1.15, 1.05, 1),
      ColorFilterGenerator.saturation(0.35),
    ]);
  }
}

// Maven: darkens images, increases shadows, and adds a slightly yellow tint overal
class MavenFilter extends ColorFilterSequence {
  MavenFilter() : super(name: "Maven") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(225, 240, 0, 0.1),
      ColorFilterGenerator.saturation(0.25),
      ColorFilterGenerator.contrast(0.05),
    ]);
  }
}

// Ginza: brightens and adds a warm glow
class GinzaFilter extends ColorFilterSequence {
  GinzaFilter() : super(name: "Ginza") {
    addSubFilters([
      ColorFilterGenerator.sepia(0.06),
      ColorFilterGenerator.brightness(0.1),
    ]);
  }
}

// Skyline: brightens to the image pop
class SkylineFilter extends ColorFilterSequence {
  SkylineFilter() : super(name: "Skyline") {
    addSubFilters([
      ColorFilterGenerator.saturation(0.35),
      ColorFilterGenerator.brightness(0.1),
    ]);
  }
}

// Dogpatch: increases the contrast, while washing out the lighter colors
class DogpatchFilter extends ColorFilterSequence {
  DogpatchFilter() : super(name: "Dogpatch") {
    addSubFilters([
      ColorFilterGenerator.contrast(0.15),
      ColorFilterGenerator.brightness(0.1),
    ]);
  }
}

// Brooklyn
class BrooklynFilter extends ColorFilterSequence {
  BrooklynFilter() : super(name: "Brooklyn") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(25, 240, 252, 0.05),
      ColorFilterGenerator.sepia(0.3),
    ]);
  }
}

// Helena: adds an orange and teal vibe
class HelenaFilter extends ColorFilterSequence {
  HelenaFilter() : super(name: "Helena") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(208, 208, 86, 0.2),
      ColorFilterGenerator.contrast(0.15),
    ]);
  }
}

// Ashby: gives images a great golden glow and a subtle vintage feel
class AshbyFilter extends ColorFilterSequence {
  AshbyFilter() : super(name: "Ashby") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 160, 25, 0.1),
      ColorFilterGenerator.brightness(0.1),
    ]);
  }
}

// Charmes: a high contrast filter, warming up colors in your image with a red tint
class CharmesFilter extends ColorFilterSequence {
  CharmesFilter() : super(name: "Charmes") {
    addSubFilters([
      ColorFilterGenerator.colorOverlay(255, 50, 80, 0.12),
      ColorFilterGenerator.contrast(0.05),
    ]);
  }
}

List<ColorFilterSequence> presetFiltersList = [
  NoFilter(),
  AddictiveBlueFilter(),
  AddictiveRedFilter(),
  AdenFilter(),
  AmaroFilter(),
  AshbyFilter(),
  BrannanFilter(),
  BrooklynFilter(),
  CharmesFilter(),
  ClarendonFilter(),
  CremaFilter(),
  DogpatchFilter(),
  EarlybirdFilter(),
  F1977Filter(),
  GinghamFilter(),
  GinzaFilter(),
  HefeFilter(),
  HelenaFilter(),
  HudsonFilter(),
  InkwellFilter(),
  JunoFilter(),
  KelvinFilter(),
  LarkFilter(),
  LoFiFilter(),
  LudwigFilter(),
  MavenFilter(),
  MayfairFilter(),
  MoonFilter(),
  NashvilleFilter(),
  PerpetuaFilter(),
  ReyesFilter(),
  RiseFilter(),
  SierraFilter(),
  SkylineFilter(),
  SlumberFilter(),
  StinsonFilter(),
  SutroFilter(),
  ToasterFilter(),
  ValenciaFilter(),
  VesperFilter(),
  WaldenFilter(),
  WillowFilter(),
  XProIIFilter(),
];
